#include<stdio.h>
#include<math.h>
#include"calculation.h"

//a$B9TL\$N9T%Y%/%H%k$H(Bb$B9T$a$N9T%Y%/%H%k$r8r49$9$k(B
static void ExchangeRowvector(Matrix *m, int a, int b);

Vector *CreateVector(int size)
{
	Vector *temp;

	if((temp = (Vector*)malloc(sizeof(Vector))) == NULL){
		fprintf(stderr, "Allocation Error\n");
		exit(1);
	}

	if((temp->v = (double*)malloc(sizeof(double)*size)) == NULL){
		fprintf(stderr, "Allocation Error\n");
		exit(1);
	}

	temp->size = size;

	return temp;
}

void FreeVector(Vector* vector)
{
	free(vector->v);
	free(vector);
}

Matrix *CreateMatrix(int width, int height)
{
	Matrix *temp;

	if((temp = (Matrix*)malloc(sizeof(Matrix))) == NULL){
		fprintf(stderr, "Allocation Error\n");
		exit(1);
	}

	if((temp->a = (double*)malloc(sizeof(double)*width*height)) == NULL){
		fprintf(stderr, "Allocation Error\n");
		exit(1);
	}

	temp->width = width;
	temp->height = height;

	return temp;
}

void FreeMatrix(Matrix* matrix)
{
	free(matrix->a);
	free(matrix);
}

int* LU(Matrix* m)
{
	int i, j, k;
	int width, height;
	int d;
	double max;
	int temp, *order;

	width = m->width;
	height = m->height;

	if((order = (int*)malloc(sizeof(int)*height)) == NULL){
		fprintf(stderr, "Allocation Error\n");
		exit(1);
	}

	for(i=0; i<height; i++) order[i] = i;

	for(i=0; i<height-1; i++){
		d = i;
		max = fabs(m->a[i*width + i]);

		for(j=i+1; j<height; j++){
			if(max < fabs(m->a[j*width + i])){
				d = j;
				max = fabs(m->a[j*width + i]);
			}
		}

		if(d!=i){
			temp = order[i];
			order[i] = order[d];
			order[d] = temp;

			ExchangeRowvector(m, i, d);
		}

		if(!(m->a[i*width + i])) continue;

		for(j=i+1; j<height; j++){
			m->a[j*width + i] /= m->a[i*width + i];
			for(k=i+1; k<width; k++){
				m->a[j*width + k] -= m->a[j*width + i]*m->a[i*width + k];
			}
		}
	}
	return order;
}

static void ExchangeRowvector(Matrix *m, int a, int b)
{
	int i;
	double temp;

	for(i=0; i<m->height; i++){
		temp = m->a[a*m->width + i];
		m->a[a*m->width + i] = m->a[b*m->width + i];
		m->a[b*m->width + i] = temp;
	}
}

void FBSubstitution(Matrix *matrix, Vector *ans)
{
	int i, j;
	int *order;
	double tempd;
	int width, height;

	width = matrix->width;
	height = matrix->height;

	if(height != ans->size || width != height){
		fprintf(stderr, "Error In FBSubstitution.\n");
		exit(1);
	}

	order = LU(matrix);

	for(i=0; i<ans->size; i++){
		if(i != order[i]){
			tempd = ans->v[order[i]];
			ans->v[order[i]] = ans->v[i];
			ans->v[i] = tempd;

			order[order[i]] = order[i];
			order[i] = i;
		}
	}

	for(i=1; i<height; i++){
		for(j=0; j<i; j++){
			ans->v[i] -= ans->v[j]*matrix->a[i*width + j];
		}
	}

	for(i=height-1; i>=0; i--){
		if(!(matrix->a[i*width + i])){
			fprintf(stderr, "Error In FBSubstitution.\n");
			exit(1);
		}

		ans->v[i] /= matrix->a[i*width + i];
		for(j=i-1; j>=0; j--){
			ans->v[j] -= ans->v[i]*matrix->a[j*width + i];
		}
	}

	free(order);
}


